import socket
import threading
import queue
import time
import turtle


x1, y1 = 0.0, 0.0
distance_a1_a2 = 1
meter2pixel = 100


def screen_init(width=900, height=800, t=turtle):
    t.setup(width, height)
    t.tracer(0)
    t.hideturtle()
    t.speed(0)


def turtle_init(t=turtle):
    t.hideturtle()
    t.speed(0)


def write_txt(x, y, txt, color, t=turtle, f=('Arial', 12, 'normal')):
    t.pencolor(color)
    t.up()
    t.goto(x, y)
    t.down()
    t.write(txt, move=False, align='left', font=f)
    t.up()


def draw_rect(x, y, w, h, color, t=turtle):
    t.pencolor(color)
    t.up()
    t.goto(x, y)
    t.down()
    t.goto(x + w, y)
    t.goto(x + w, y + h)
    t.goto(x, y + h)
    t.goto(x, y)
    t.up()


# =("black", "black")
def fill_rect(x, y, w, h, color, t=turtle):
    t.begin_fill()
    draw_rect(x, y, w, h, color, t)
    t.end_fill()
    pass


def clean(t=turtle):
    t.clear()


def fill_cycle(x, y, r, color, t=turtle):
    t.up()
    t.goto(x, y)
    t.down()
    t.dot(r, color)
    t.up()


def map_to_range(val, min_val, max_val, new_min, new_max):
    # 将一个值从给定的范围映射到新的范围。
    return ((val - min_val) / (max_val - min_val)) * (new_max - new_min) + new_min


def draw_ui(t):
    write_txt(-200, 350, "ELECORW UWB1000", "blue", t, f=('Arial', 26, 'normal'))
    # fill_rect(-600, 330, 1200, 20, "black", t)
    t.pencolor("black")


def draw_uwb_anchor(x, y, txt, range, t):
    r = 10
    fill_cycle(x, y, r, "red", t)
    write_txt(x + r, y, txt,
              "black", t, f=('Arial', 12, 'normal'))


def draw_uwb_tag(xt, yt, txt, t):
    pos_x = -280 + int(xt * meter2pixel)
    pos_y = 200 - int(yt * meter2pixel)
    r = 10
    x_mapped = map_to_range(pos_x, -280, -280 + meter2pixel * distance_a1_a2, -280, 220)
    y_mapped = map_to_range(pos_y, 200 - meter2pixel * distance_a1_a2, 200, -300, 200)
    # print("x_mapped=", x_mapped, ",y_mapped=", y_mapped)
    # t_ui.pencolor("black")

    fill_cycle(x_mapped, y_mapped, r, "blue", t)
    write_txt(x_mapped, y_mapped, txt + ": (" + str(xt) + "," + str(yt) + ")",
              "black", t, f=('Arial', 12, 'normal'))



localIP = "172.20.10.2"  # Python服务器的IP地址
localPort = 12000     # Python服务器的端口号

bufferSize = 1024

# 创建UDP套接字
udpSocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

# 绑定地址和端口
udpSocket.bind((localIP, localPort))
print("UDP server listening on {}:{}".format(localIP, localPort))

turtle.title("ELECROW UWB1000")
screen_init()

t_ui = turtle.Turtle()
t_a1 = turtle.Turtle()
t_a2 = turtle.Turtle()
t_a3 = turtle.Turtle()
t_a4 = turtle.Turtle()
t_a5 = turtle.Turtle()

turtle_init(t_ui)
turtle_init(t_a1)
turtle_init(t_a2)
turtle_init(t_a3)
turtle_init(t_a4)
turtle_init(t_a5)
draw_ui(t_ui)

# 绘制直线
t_ui.goto(-280, 200)
t_ui.pendown()
t_ui.goto(-280+ meter2pixel * 5,  200)
t_ui.penup()
t_ui.goto(-280, 200)
t_ui.pendown()
t_ui.goto(-280, 200-meter2pixel * 5)
t_ui.penup()

draw_uwb_anchor(-280, 200, "A0(0,0)", x1, t_a1)
draw_uwb_anchor(-280+ meter2pixel * 5,  200, "A1(" + str(distance_a1_a2) + ",0)", y1, t_a2)
draw_uwb_anchor(-280, 200-meter2pixel * 5 , "A2(0," + str(distance_a1_a2) + ")", x1, t_a3)

ip_addresses1 = ""
ip_addresses2 = ""
ip_addresses3 = ""
while True:
    # 接收数据
    data, address = udpSocket.recvfrom(bufferSize)
    print("Received data from {}: {}".format(address, data.decode()))
    values = data.decode().split(",")
    if len(values) == 2:
        x1 = float(values[0])
        y1 = float(values[1])
    print("x：", x1)
    print("y：", y1)
    # print("address：", address[0])
    current_connection_ip = address[0]
    if current_connection_ip != ip_addresses1 and current_connection_ip != ip_addresses2 and current_connection_ip != ip_addresses3:
        if ip_addresses1 == "":
            ip_addresses1 = current_connection_ip
            print("将第一个IP地址保存到 ip_addresses1")
        elif ip_addresses2 == "":
            ip_addresses2 = current_connection_ip
            print("将第二个IP地址保存到 ip_addresses2")
        elif ip_addresses3 == "":
            ip_addresses3 = current_connection_ip
            print("将第三个IP地址保存到 ip_addresses3")
    print(ip_addresses1)
    print(ip_addresses2)
    print(ip_addresses3)
    # clean(t_a4)
    # clean(t_a5)
    if current_connection_ip == ip_addresses1:
        clean(t_a4)
        draw_uwb_tag(x1, y1, ip_addresses1, t_a4)
    if current_connection_ip == ip_addresses2:
        clean(t_a5)
        draw_uwb_tag(x1, y1, ip_addresses2, t_a5)
    # draw_uwb_tag(x1, y1, "TAG", t_a4)